@@ -20,6 +20,7 @@
 *****************************************************************************/
 
 #include "samplv1widget.h"
+#include "samplv1_param.h"
 
 #include "samplv1_sample.h"
 
@@ -31,81 +32,6 @@
 #include <QDir>
 
 
-//-------------------------------------------------------------------------
-// default state (params)
-
-static
-struct {
-
-	const char *name;
-	float value;
-
-} samplv1_default_params[samplv1::NUM_PARAMS] = {
-
-	{ "GEN1_SAMPLE",   60.0f }, // middle-C aka. C4 (60)
-	{ "GEN1_LOOP",      0.0f },
-	{ "GEN1_OCTAVE",    0.0f },
-	{ "GEN1_TUNING",    0.0f },
-	{ "GEN1_GLIDE",     0.0f },
-	{ "DCF1_CUTOFF",    1.0f }, // 0.5f
-	{ "DCF1_RESO",      0.0f },
-	{ "DCF1_TYPE",      0.0f },
-	{ "DCF1_SLOPE",     0.0f },
-	{ "DCF1_ENVELOPE",  1.0f },
-	{ "DCF1_ATTACK",    0.0f },
-	{ "DCF1_DECAY",     0.2f },
-	{ "DCF1_SUSTAIN",   0.5f },
-	{ "DCF1_RELEASE",   0.5f },
-	{ "LFO1_SHAPE",     1.0f },
-	{ "LFO1_WIDTH",     1.0f },
-	{ "LFO1_RATE",      0.5f },
-	{ "LFO1_SWEEP",     0.0f },
-	{ "LFO1_PITCH",     0.0f },
-	{ "LFO1_CUTOFF",    0.0f },
-	{ "LFO1_RESO",      0.0f },
-	{ "LFO1_PANNING",   0.0f },
-	{ "LFO1_VOLUME",    0.0f },
-	{ "LFO1_ATTACK",    0.0f },
-	{ "LFO1_DECAY",     0.1f },
-	{ "LFO1_SUSTAIN",   1.0f },
-	{ "LFO1_RELEASE",   0.5f },
-	{ "DCA1_VOLUME",    0.5f },
-	{ "DCA1_ATTACK",    0.0f },
-	{ "DCA1_DECAY",     0.1f },
-	{ "DCA1_SUSTAIN",   1.0f },
-	{ "DCA1_RELEASE",   0.5f },	// 0.1f
-	{ "OUT1_WIDTH",     0.0f },
-	{ "OUT1_PANNING",   0.0f },
-	{ "OUT1_VOLUME",    0.5f },
-
-	{ "DEF1_PITCHBEND", 0.2f },
-	{ "DEF1_MODWHEEL",  0.2f },
-	{ "DEF1_PRESSURE",  0.2f },
-	{ "DEF1_VELOCITY",  0.2f },
-
-	{ "CHO1_WET",       0.0f },
-	{ "CHO1_DELAY",     0.5f },
-	{ "CHO1_FEEDB",     0.5f },
-	{ "CHO1_RATE",      0.5f },
-	{ "CHO1_MOD",       0.5f },
-	{ "FLA1_WET",       0.0f },
-	{ "FLA1_DELAY",     0.5f },
-	{ "FLA1_FEEDB",     0.5f },
-	{ "FLA1_DAFT",      0.0f },
-	{ "PHA1_WET",       0.0f },
-	{ "PHA1_RATE",      0.5f },
-	{ "PHA1_FEEDB",     0.5f },
-	{ "PHA1_DEPTH",     0.5f },
-	{ "PHA1_DAFT",      0.0f },
-	{ "DEL1_WET",       0.0f },
-	{ "DEL1_DELAY",     0.5f },
-	{ "DEL1_FEEDB",     0.5f },
-	{ "DEL1_BPM",     180.0f },
-	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
-};
-
-
 //-------------------------------------------------------------------------
 // samplv1widget - impl.
 //
@@ -139,7 +65,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
-		m_params_ab[i] = samplv1_default_params[i].value;
+		m_params_ab[i] = samplv1_param::paramDefaultValue(samplv1::ParamIndex(i));
 
 	// Start clean.
 	m_iUpdate = 0;
@@ -248,6 +174,17 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Lfo1VolumeKnob->setMinimum(-1.0f);
 	m_ui.Lfo1VolumeKnob->setMaximum(+1.0f);
 
+	// Channel filters
+	QStringList channels;
+	channels << tr("Omni");
+	for (int iChannel = 0; iChannel < 16; ++iChannel)
+		channels << QString::number(iChannel + 1);
+
+	m_ui.Def1ChannelKnob->insertItems(0, channels);
+
+	// Mono switches
+	m_ui.Def1MonoKnob->insertItems(0, states);
+
 	// Output (stereo-)width limits.
 	m_ui.Out1WidthKnob->setMinimum(-1.0f);
 	m_ui.Out1WidthKnob->setMaximum(+1.0f);
@@ -425,6 +362,8 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(samplv1::DEF1_MODWHEEL,  m_ui.Def1ModwheelKnob);
 	setParamKnob(samplv1::DEF1_PRESSURE,  m_ui.Def1PressureKnob);
 	setParamKnob(samplv1::DEF1_VELOCITY,  m_ui.Def1VelocityKnob);
+	setParamKnob(samplv1::DEF1_CHANNEL,   m_ui.Def1ChannelKnob);
+	setParamKnob(samplv1::DEF1_MONO,      m_ui.Def1MonoKnob);
 
 	// OUT1
 	setParamKnob(samplv1::OUT1_WIDTH,   m_ui.Out1WidthKnob);
@@ -615,11 +554,17 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 // Reset all param knobs to default values.
 void samplv1widget::resetParams (void)
 {
+	samplv1 *pSampl = instance();
+	if (pSampl == NULL)
+		return;
+
+	pSampl->reset();
+
 	resetSwapParams();
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		float fValue = samplv1_default_params[i].value;
+		float fValue = samplv1_param::paramDefaultValue(index);
 		samplv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob)
 			fValue = pKnob->defaultValue();
@@ -678,7 +623,7 @@ void samplv1widget::resetParamValues (void)
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		float fValue = samplv1_default_params[i].value;
+		float fValue = samplv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
@@ -743,8 +688,10 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 
 	static QHash<QString, samplv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
-		for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i)
-			s_hash.insert(samplv1_default_params[i].name, samplv1::ParamIndex(i));
+		for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
+			samplv1::ParamIndex index = samplv1::ParamIndex(i);
+			s_hash.insert(samplv1_param::paramName(index), index);
+		}
 	}
 
 	clearSampleFile();
@@ -830,10 +777,11 @@ void samplv1widget::savePreset ( const QString& sFilename )
 	QDomElement eParams = doc.createElement("params");
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		QDomElement eParam = doc.createElement("param");
+		samplv1::ParamIndex index = samplv1::ParamIndex(i);
 		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", samplv1_default_params[i].name);
-		eParam.appendChild(doc.createTextNode(QString::number(
-			paramValue(samplv1::ParamIndex(i)))));
+		eParam.setAttribute("name", samplv1_param::paramName(index));
+		eParam.appendChild(
+			doc.createTextNode(QString::number(paramValue(index))));
 		eParams.appendChild(eParam);
 	}
 	ePreset.appendChild(eParams);
